@@ -1,17 +1,17 @@
 # vim: ts=2 sw=2 expandtab
 # -*- coding: utf-8 -*-
 # Copyright © 2011, Lucas Nussbaum <lucas@debian.org>
-# 
+#
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -60,7 +60,11 @@ module Gem2Deb
 
     attr_accessor :extra_build_dependencies
 
+    attr_accessor :overwrite
+
     def initialize(input, options = {})
+      generate_or_update_gem_to_package_data
+
       initialize_from_options(options)
       if File.directory?(input)
         initialize_from_directory(input)
@@ -82,7 +86,16 @@ module Gem2Deb
       read_metadata(directory)
       self.gem_name = metadata.name
       self.gem_version = metadata.version
-      self.source_package_name ||= gem_name_to_source_package_name(gem_name)
+      self.source_package_name ||= get_source_package_name(directory)
+    end
+
+    def get_source_package_name(directory)
+      changelog = File.join(directory, 'debian/changelog')
+      if File.exist?(changelog)
+        `dpkg-parsechangelog -l#{changelog} -SSource`.strip
+      else
+        gem_name_to_source_package_name(gem_name)
+      end
     end
 
     def initialize_from_tarball(tarball)
@@ -105,9 +118,54 @@ module Gem2Deb
     end
 
     def gem_name_to_source_package_name(gem_name)
-      'ruby-' + gem_name.gsub(/^ruby[-_]|[-_]ruby$/, '')
+      @gem_to_package[gem_name] || 'ruby-' + gem_name.downcase.gsub(/^ruby[-_]|[-_]ruby$/, '').gsub('_', '-')
     end
 
+    def generate_or_update_gem_to_package_data
+      if Gem2Deb.testing
+        @gem_to_package = { 'rake' => 'rake', 'rails' => 'rails' }
+        return
+      end
+
+      if !File.exists?('/usr/bin/apt-file')
+        puts "E: apt-file not found. Please install the package apt-file"
+        exit 1
+      end
+
+      cache_dir = File.join(ENV['HOME'], '.cache', 'gem2deb')
+      FileUtils.mkdir_p(cache_dir)
+      cache = File.join(cache_dir, 'gem_to_packages.yaml')
+
+      if File.exists?(cache)
+        stat = File.stat(cache)
+        update = (Time.now.to_i - stat.mtime.to_i) > (60*60*24) # keep cache for 24h
+      else
+        update = true
+      end
+
+      if update
+        if system('apt-file search /usr/share/rubygems-integration/ > ' + cache + '.new')
+          system('sed', '-i', '-e', 's#/.*/##; s/-[0-9.]\+.gemspec//', cache + '.new')
+          FileUtils.mv(cache + '.new', cache)
+        else
+          puts 'E: dh-make-ruby needs an up-to-date apt-file cache in order to map gem names to package names'
+          exit $?.exitstatus
+        end
+      end
+
+      data = YAML.load_file(cache)
+      unless data.respond_to?(:invert)
+        File.unlink(cache)
+        puts 'E: Failed to load "gem name to package name" cache from'
+        puts '   ' +  cache
+        puts 'I: The existing cache was removed and will be rebuilt next time.'
+        puts 'I: please try again.'
+        exit 1
+      end
+      @gem_to_package = data.invert
+    end
+
+
     def gem_dirname
       [gem_name, gem_version].join('-')
     end
@@ -144,13 +202,21 @@ module Gem2Deb
     def build_in_directory(directory)
       Dir.chdir(directory) do
         read_upstream_source_info
+        previously_debianized = File.directory?('debian')
         FileUtils.mkdir_p('debian')
         other_files
         test_suite
         create_debian_boilerplates
+        if overwrite || !previously_debianized
+          wrap_and_sort
+        end
       end
     end
-    
+
+    def wrap_and_sort
+      run('wrap-and-sort', '--wrap-always')
+    end
+
     def read_upstream_source_info
       read_metadata('.')
       initialize_binary_package
@@ -162,12 +228,29 @@ module Gem2Deb
 
     def initialize_binary_package
       self.binary_package = Package.new(source_package_name, metadata.has_native_extensions? ? 'any' : 'all')
-      metadata.dependencies.each do |dependency|
-        binary_package.gem_dependencies << dependency
+      with_each_runtime_dependency do |dependency|
+        binary_package.dependencies << dependency
       end
       binary_package
     end
 
+    def with_each_runtime_dependency
+      (metadata.dependencies).select do |dep|
+        dep.type == :runtime
+      end.each do |dep|
+        dependency = gem_name_to_source_package_name(dep.name)
+        version = dep.requirement.to_s
+        if version == '>= 0'
+          yield(dependency)
+        else
+          dep.requirements_list.each do |v|
+            spec = v.gsub('~>', '>=').gsub(/>(\s+)/, '>>\1').gsub(/<(\s+)/, '<<\1').gsub(/^=(\s+)/, '>=\1')
+            yield('%s (%s)' % [dependency, spec])
+          end
+        end
+      end
+    end
+
     def buildpackage(source_only = false, check_build_deps = true)
       dpkg_buildpackage_opts = []
       dpkg_buildpackage_opts << '-S' if source_only
@@ -206,10 +289,11 @@ module Gem2Deb
       end
     end
 
-    def write_if_missing(filename)
-      unless File.exist? filename
-        File.open(filename, 'w') { |f| yield f }
+    def maybe_create(filename)
+      if File.exist?(filename) && !overwrite
+        return
       end
+      File.open(filename, 'w') { |f| yield f }
     end
 
     def create_debian_boilerplates
@@ -221,7 +305,7 @@ module Gem2Deb
       end
       templates.each do |template|
         FileUtils.mkdir_p(template.directory)
-        write_if_missing(template.filename) do |f|
+        maybe_create(template.filename) do |f|
           f.puts ERB.new(template.data, nil, '<>').result(binding)
         end
       end
@@ -292,31 +376,24 @@ module Gem2Deb
         self.architecture = architecture
       end
       def dependencies
-        ['${shlibs:Depends}', '${misc:Depends}', 'ruby | ruby-interpreter' ]
-      end
-      def gem_dependencies
-	@gem_dependencies ||= []
+        @dependencies ||= []
       end
     end
 
     def test_suite
-      if !Gem2Deb::TestRunner.detect_real_runner
-        test_suite_rspec or test_suite_yaml or test_suite_rb
-      end
+      test_suite_rspec or test_suite_yaml or test_suite_rb
     end
 
     def test_suite_rspec
       if File::directory?("spec")
         extra_build_dependencies << 'ruby-rspec' << 'rake'
-        write_if_missing("debian/ruby-tests.rake") do |f|
+        maybe_create("debian/ruby-tests.rake") do |f|
           f.puts <<-EOF
-require 'rspec/core/rake_task'
+require 'gem2deb/rake/spectask'
 
-RSpec::Core::RakeTask.new(:spec) do |spec|
-  spec.pattern = './spec/*_spec.rb'
+Gem2Deb::Rake::RSpecTask.new do |spec|
+  spec.pattern = './spec/**/*_spec.rb'
 end
-
-task :default => :spec
         EOF
         end
         true
@@ -328,7 +405,7 @@ task :default => :spec
 
     def test_suite_yaml
       if !metadata.test_files.empty?
-        write_if_missing("debian/ruby-test-files.yaml") do |f|
+        maybe_create("debian/ruby-test-files.yaml") do |f|
           YAML::dump(metadata.test_files, f)
         end
         true
@@ -340,12 +417,13 @@ task :default => :spec
     def test_suite_rb
       if File::directory?("test")
         extra_build_dependencies << 'rake'
-        write_if_missing("debian/ruby-tests.rake") do |f|
+        maybe_create("debian/ruby-tests.rake") do |f|
           f.puts <<-EOF
 require 'gem2deb/rake/testtask'
 
 Gem2Deb::Rake::TestTask.new do |t|
-  t.test_files = FileList['test/*_test.rb']
+  t.libs = ['test']
+  t.test_files = FileList['test/**/*_test.rb'] + FileList['test/**/test_*.rb']
 end
         EOF
         end
@@ -367,15 +445,11 @@ end
             EOF
       end
       readmes = Dir::glob('README*')
-      docs += <<-EOF
-# FIXME: READMEs found
-      EOF
-      readmes.each do |r|
-        docs << "# #{r}\n"
-      end
-      if docs != ""
-        write_if_missing("debian/#{source_package_name}.docs") do |f|
-          f.puts docs
+      if !readmes.empty?
+        maybe_create("debian/#{source_package_name}.docs") do |f|
+          readmes.each do |r|
+            f.puts r
+          end
         end
       end
 
@@ -391,7 +465,7 @@ end
         end
       end
       if examples != ""
-        write_if_missing("debian/#{source_package_name}.examples") do |f|
+        maybe_create("debian/#{source_package_name}.examples") do |f|
           f.puts examples
         end
       end
@@ -413,7 +487,7 @@ end
         EOF
       end
       if installs != ""
-        write_if_missing("debian/#{source_package_name}.install") do |f|
+        maybe_create("debian/#{source_package_name}.install") do |f|
           f.puts installs
         end
       end
@@ -423,7 +497,7 @@ end
         manpages = Dir.glob("man/**/*.[1-8]")
         manpages_header = "# FIXME: man/ dir found in source. Consider installing manpages"
 
-        write_if_missing("debian/#{source_package_name}.manpages") do |f|
+        maybe_create("debian/#{source_package_name}.manpages") do |f|
           f.puts manpages_header
           manpages.each do |m|
             f.puts "# " + m
