@@ -1,11 +1,10 @@
+require 'gem2deb/banner'
 require 'gem2deb/metadata'
 
 module Gem2Deb
 
   class Installer
 
-    class RequireRubygemsFound < Exception; end
-
     include Gem2Deb
 
     EXTENSION_BUILDER = File.expand_path(File.join(File.dirname(__FILE__),'extension_builder.rb'))
@@ -16,7 +15,7 @@ module Gem2Deb
 
     attr_reader :ruby_versions
     attr_accessor :verbose
-    attr_accessor :dh_auto_install_destdir
+    attr_accessor :destdir_base
 
     def initialize(binary_package, root, ruby_versions = SUPPORTED_RUBY_VERSIONS.keys)
       @binary_package = binary_package
@@ -26,14 +25,16 @@ module Gem2Deb
     end
 
     def install_files_and_build_extensions
-      install_files(bindir, destdir(:bindir), 755) if File::directory?(bindir)
 
+      Gem2Deb::Banner.print "Install files"
+      install_files(bindir, destdir(:bindir), 755) if File::directory?(bindir)
       install_files(libdir, destdir(:libdir), 644) if File::directory?(libdir)
 
       if metadata.has_native_extensions?
         ruby_versions.each do |rubyver|
-          puts "Building extension for #{rubyver} ..." if verbose
-          run(SUPPORTED_RUBY_VERSIONS[rubyver], "-I#{LIBDIR}", EXTENSION_BUILDER, root, destdir_base)
+          Gem2Deb::Banner.print "Build native extensions for #{rubyver}" if verbose
+
+          run_ruby(SUPPORTED_RUBY_VERSIONS[rubyver], EXTENSION_BUILDER, root, destdir_base)
 
           # Remove duplicate files installed by rubygems in the arch dir
           # This is a hack to workaround a problem in rubygems
@@ -71,16 +72,17 @@ module Gem2Deb
     end
 
     def install_gemspec
+      Gem2Deb::Banner.print "Install Rubygems integration metadata"
       if metadata.gemspec
         versions =
           if metadata.has_native_extensions?
-            ruby_versions.map { |v| RUBY_CONFIG_VERSION[v] }
+            ruby_versions.map { |v| RUBY_API_VERSION[v] }
           else
             ['all']
           end
         versions.each do |version|
           target = File.join(destdir(:root), "/usr/share/rubygems-integration/#{version}/specifications/#{metadata.name}-#{metadata.version}.gemspec")
-          puts "generating gemspec at ${target}"
+          puts "generating gemspec at #{target}"
           FileUtils.mkdir_p(File.dirname(target))
           File.open(target, 'w') do |file|
             file.write(metadata.gemspec.to_ruby)
@@ -89,29 +91,6 @@ module Gem2Deb
       end
     end
 
-    def check_rubygems
-      found = false
-      if File::exists?('debian/require-rubygems.overrides')
-        overrides = YAML::load_file('debian/require-rubygems.overrides')
-      else
-        overrides = []
-      end
-      installed_ruby_files.each do |f|
-        lines = readlines(f)
-        rglines = lines.select { |l| l =~ /require.*rubygems/  && l !~ /^\s*#/ }
-          rglines.each do |l|
-          if not overrides.include?(f)
-            puts "#{f}: #{l}" if verbose
-            found = true
-          end
-          end
-      end
-      if found
-        puts "Found some 'require rubygems' without overrides (see above)." if verbose
-        raise RequireRubygemsFound
-      end
-    end
-
     def run_make_clean_on_extensions
       if metadata.has_native_extensions?
         metadata.native_extensions.each do |extension|
@@ -169,14 +148,6 @@ module Gem2Deb
       end
     end
 
-    def destdir_base
-      if ENV['DH_RUBY_USE_DH_AUTO_INSTALL_DESTDIR']
-        self.dh_auto_install_destdir
-      else
-        File.join('debian', binary_package)
-      end
-    end
-
     JUNK_FILES = %w( RCSLOG tags TAGS .make.state .nse_depinfo )
     HOOK_FILES = %w( pre-%s post-%s pre-%s.rb post-%s.rb ).map {|fmt|
       %w( config setup install clean ).map {|t| sprintf(fmt, t) }
