@@ -15,6 +15,10 @@
 
 require 'rbconfig'
 require 'fileutils'
+require 'shellwords'
+
+require 'gem2deb/banner'
+require 'gem2deb/metadata'
 
 module Gem2Deb
   class TestRunner
@@ -22,6 +26,7 @@ module Gem2Deb
     include FileUtils::Verbose
 
     attr_accessor :autopkgtest
+    attr_accessor :check_dependencies
 
     def load_path
       if self.autopkgtest
@@ -39,7 +44,7 @@ module Gem2Deb
 
       dirs = []
       $LOAD_PATH.grep(/vendor/).each do |dir|
-        dirs += Dir.glob('debian/*/' + dir)
+        dirs += Dir.glob('debian/*/' + dir).map { |d| File.expand_path(d) }
       end
 
       # And we add the current directory:
@@ -48,8 +53,40 @@ module Gem2Deb
       dirs
     end
 
-    # Override in subclasses
+    def env_with_gem_path
+      if self.autopkgtest
+        { }
+      else
+        {
+          'GEM_PATH' => (Gem.path + Dir.glob("debian/*/usr/share/rubygems-integration/{all,#{ruby_api_version}}")).join(':')
+        }
+      end
+    end
+
     def run_tests
+      if check_dependencies
+        do_check_dependencies
+      end
+      do_run_tests
+    end
+
+    def do_check_dependencies
+      print_banner "Checking Rubygems dependency resolution on #{rubyver}"
+      metadata = Gem2Deb::Metadata.new('.')
+      if metadata.gemspec
+        cmd = [rubyver, '-e', 'gem "%s"' % metadata.name]
+        puts "GEM_PATH=#{env_with_gem_path['GEM_PATH']} " + cmd.shelljoin
+        system(env_with_gem_path, *cmd)
+        exitstatus = $?.exitstatus
+        if exitstatus != 0
+          system 'gem', 'list'
+          exit(exitstatus)
+        end
+      end
+    end
+
+    # Override in subclasses
+    def do_run_tests
     end
 
     # override in subclasses
@@ -57,28 +94,30 @@ module Gem2Deb
       nil
     end
 
-    # override in subclasses if needed
-    def real_runner?
-      true
-    end
-
     def activate?
       required_file && File.exist?(required_file)
     end
 
-    def run_ruby(*cmd)
+    def run_ruby(*args)
+      run(rubyver, *args)
+    end
+
+    def run_rake(*args)
+      run(rubyver.sub('ruby', 'rake'), *args)
+    end
+
+    def run(program, *args)
       rubylib = load_path.join(':')
-      cmd.unshift(rubyver)
-      if $VERBOSE
-        print "RUBYLIB=#{rubylib} "
-        puts cmd.map { |part| part =~ /['"]/ ? part.inspect : part }.join(' ')
-      end
-      ENV['RUBYLIB'] = (ENV['RUBYLIB'] ? ENV['RUBYLIB'] + ':' : '') + rubylib
+      cmd = [program] + args
+
+      rlib = (ENV['RUBYLIB'] ? ENV['RUBYLIB'] + ':' : '') + rubylib
+      puts "RUBYLIB=#{rubylib} " + cmd.shelljoin
+
       if autopkgtest
         move_away 'lib'
         move_away 'ext'
       end
-      system(*cmd)
+      system({ 'RUBYLIB' => rlib }, *cmd)
       exitstatus = $?.exitstatus
       if autopkgtest
         restore 'lib'
@@ -114,11 +153,6 @@ module Gem2Deb
     def self.detect!
       detect || bail("E: this tool must be run from inside a Debian source package.")
     end
-    def self.detect_real_runner
-      subclasses.map(&:new).find do |runner|
-        runner.real_runner? && runner.activate?
-      end
-    end
     def self.bail(msg)
       puts msg
       exit 1
@@ -126,18 +160,25 @@ module Gem2Deb
     def rubyver
       @rubyver ||= RbConfig::CONFIG['ruby_install_name']
     end
+    def ruby_api_version
+      RbConfig::CONFIG['ruby_version']
+    end
     def ruby_binary
       @ruby_binary ||= File.join('/usr/bin', rubyver)
     end
 
+    def print_banner(msg)
+      Gem2Deb::Banner.print(msg)
+    end
+
     class TestsListedInMetadata < TestRunner
       def required_file
         'debian/ruby-test-files.yaml'
       end
-      def run_tests
-        puts "Running tests for #{rubyver} with test file list from debian/ruby-test-files.yaml ..."
+      def do_run_tests
+        print_banner "Run tests for #{rubyver} from debian/ruby-test-files.yaml"
         run_ruby(
-          '-ryaml', 
+          '-ryaml',
           '-e',
           'YAML.load_file("debian/ruby-test-files.yaml").each { |f| require f }'
         )
@@ -148,13 +189,9 @@ module Gem2Deb
       def required_file
         'debian/ruby-tests.rake'
       end
-      def run_tests
-        puts "Running tests for #{rubyver} using debian/ruby-tests.rake ..."
-        run_ruby(
-          '-rrake',
-          '-e',
-          'ARGV.unshift("-f", "debian/ruby-tests.rake"); Rake.application.run'
-        )
+      def do_run_tests
+        print_banner "Run tests for #{rubyver} from debian/ruby-tests.rake"
+        run_rake('-f', 'debian/ruby-tests.rake')
       end
     end
 
@@ -162,8 +199,8 @@ module Gem2Deb
       def required_file
         'debian/ruby-tests.rb'
       end
-      def run_tests
-        puts "Running tests for #{rubyver} using debian/ruby-tests.rb..."
+      def do_run_tests
+        print_banner "Run tests for #{rubyver} from debian/ruby-tests.rb"
         ENV['RUBY_TEST_VERSION'] = rubyver
         ENV['RUBY_TEST_BIN'] = ruby_binary
         run_ruby(required_file)
@@ -174,11 +211,8 @@ module Gem2Deb
       def required_file
         'debian/rules'
       end
-      def run_tests
-        puts "Running tests for #{rubyver}: found no way to run a test suite!"
-      end
-      def real_runner?
-        false
+      def do_run_tests
+        print_banner "Run tests for #{rubyver}: no test suite!"
       end
     end
 
