@@ -1,3 +1,8 @@
+unless ENV['ADTTMP']
+  $LOAD_PATH.unshift File.expand_path('../../lib', __FILE__)
+end
+
+
 require 'test/unit'
 require 'shoulda-context'
 require 'mocha/setup'
@@ -6,11 +11,14 @@ require 'tmpdir'
 require 'tempfile'
 
 require 'gem2deb'
+Gem2Deb.verbose = false
+Gem2Deb.testing = true
 
 Gem2DebTestCase = Test::Unit::TestCase
 class Gem2DebTestCase
 
   SUPPORTED_VERSION_NUMBERS = Gem2Deb::RUBY_CONFIG_VERSION.values.sort
+  SUPPORTED_API_NUMBERS = Gem2Deb::RUBY_API_VERSION.values.sort
 
   OLDER_RUBY_VERSION = Gem2Deb::SUPPORTED_RUBY_VERSIONS.keys.select { |m| m =~ /^ruby/ }.sort.first
   OLDER_RUBY_VERSION_BINARY = Gem2Deb::SUPPORTED_RUBY_VERSIONS[OLDER_RUBY_VERSION]
@@ -21,7 +29,7 @@ class Gem2DebTestCase
       `#{Gem2Deb::SUPPORTED_RUBY_VERSIONS[version]} -rrbconfig -e "puts RbConfig::CONFIG['vendorarchdir']"`.strip
   end
 
-  require 'test_helper/samples'
+  require_relative 'test_helper/samples'
   include Gem2DebTestCase::Samples
 
   GEM2DEB_ROOT_SOURCE_DIR = File.expand_path(File.join(File.dirname(__FILE__), '..'))
