@@ -1,4 +1,4 @@
-require 'test_helper'
+require_relative '../test_helper'
 require 'gem2deb/gem2tgz'
 require 'gem2deb/dh_make_ruby'
 
@@ -17,6 +17,11 @@ class DhMakeRubyTest < Gem2DebTestCase
     assert_equal 'ruby-simplegem', Gem2Deb::DhMakeRuby.new(SIMPLE_GEM_UPSTREAM_TARBALL).source_package_name
   end
 
+  should 'use existing package name if present' do
+    dmr = Gem2Deb::DhMakeRuby.new(KILLERAPP_DIR)
+    assert_equal 'killerapp', dmr.source_package_name
+  end
+
   should 'be able to specify a package name' do
     assert_equal 'xyz', Gem2Deb::DhMakeRuby.new(SIMPLE_GEM_UPSTREAM_TARBALL, :source_package_name => 'xyz').source_package_name
   end
@@ -30,6 +35,14 @@ class DhMakeRubyTest < Gem2DebTestCase
     assert_equal 'ruby-foo', Gem2Deb::DhMakeRuby.new('foo_ruby-1.2.3.tar.gz').source_package_name
   end
 
+  should 'properly convert CFPropertyList to debian package name' do
+    assert_equal 'ruby-cfpropertylist', Gem2Deb::DhMakeRuby.new('CFPropertyList-1.2.3.tar.gz').source_package_name
+  end
+
+  should 'properly convert Fancy_Package to debian package name' do
+    assert_equal 'ruby-fancy-package', Gem2Deb::DhMakeRuby.new('Fancy_Package-1.2.3.tar.gz').source_package_name
+  end
+
   should 'use #nnnn if no ITP bug exists' do
       @dh_make_ruby = Gem2Deb::DhMakeRuby.new('ruby_foo-1.2.3.tar.gz', :do_wnpp_check => true)
       @dh_make_ruby.stubs(:wnpp_check).returns('')
@@ -124,6 +137,43 @@ class DhMakeRubyTest < Gem2DebTestCase
     end
   end
 
+  context 'dependencies' do
+    setup do
+      file = File.join(DEBIANIZED_SIMPLE_GEM, 'debian/control')
+      deps = `grep-dctrl -sDepends -n '' #{file}`
+      @dependencies = deps.strip.split(/\s*,\s*/)
+    end
+    should 'get simple dependency' do
+      assert_include @dependencies, 'ruby-dep'
+    end
+    should 'not use dependencies with exact versions' do
+      assert_include @dependencies, 'ruby-depwithversion (>= 1.0)'
+    end
+    should 'get version with spermy' do
+      assert_include @dependencies, 'ruby-depwithspermy (>= 1.0)'
+    end
+    should 'get version with >' do
+      assert_include @dependencies, 'ruby-depwithgt (>> 1.0)'
+    end
+    should 'get version with two requirements' do
+      assert_include @dependencies, 'ruby-depwith2versions (>= 1.0)'
+      assert_include @dependencies, 'ruby-depwith2versions (<< 2.0)'
+    end
+  end
+
+  context 'converting gem name to package name without a cache' do
+    {
+      'foo' => 'ruby-foo',
+      'foo-bar_baz' => 'ruby-foo-bar-baz',
+    }.each do |input,output|
+      setup do
+        @dh_make_ruby = Gem2Deb::DhMakeRuby.new('.')
+      end
+      should "convert #{input} to #{output}" do
+        assert_equal output, @dh_make_ruby.gem_name_to_source_package_name(input)
+      end
+    end
+  end
 
   protected
 
